function d2fdy2  = nylond2fdy2(t,fd_cell,params,fn_extras)
% Nylon612 equations 
% second derivatives with respect to outputs
%
% This is a 4 parameter version of the model from:
% Wei Zheng, Kim B. McAuley, E. Keith Marchildon and
% Yao K. Zhen, 2005, "Effects of End-Group Balance on Melt-Phase Nylon 612
% Polycondensation: Experimental Study and Mathematical Model", Ind. Eng. Chem. Res.
%  vol 44, p 2675 - 2686
%
%
% y has columns A C W and rows are for time
% 
%pars = [K_p0, gamma, Ka0,  deltaH ]
% 
constants    = [549, 24.3, 20.97, 647.3, 165376,8.3145e-3];
% constants = [T_0, k_m,gamma_w0,Tc,Pc,R]
%                       1    2       3             4   5  6
T = fn_extras{1};
W_eq = eval_fd(t,fn_extras{2});

exp_multiplier=1/constants(6)*(1/T-1/constants(1));
kp=params(1)/1000;
gamma_w = exp(9.624-3613/T);
K_a = (1+params(2)/1e3*W_eq)*(constants(3)/gamma_w)*params(3)*exp(-params(4)*exp_multiplier);

dLdA = (-58.10)/155.23;
dLdC = (- 115.15)/155.23;
dLdW = (-18.02)/155.23;

d2fdy2=cell(3,3,3);

d2fdy2{1,1,1}=0;
d2fdy2{1,1,2}=-kp;
d2fdy2{1,1,3}=kp*dLdA./K_a;

d2fdy2{1,2,1}=-kp;
d2fdy2{1,2,2}=0;
d2fdy2{1,2,3}=kp.*dLdC./K_a;

d2fdy2{1,3,1}=kp./K_a.*(dLdA);
d2fdy2{1,3,2}=kp./K_a.*(dLdC);
d2fdy2{1,3,3}=kp./K_a.*(dLdW*2);

d2fdy2(2,:,:)=d2fdy2(1,:,:);

for i=1:size(d2fdy2,2)
    for j=1:size(d2fdy2,3)
        d2fdy2{3,i,j}=-d2fdy2{1,i,j};
    end
end
